@@ -2,8 +2,8 @@
  *
  *  BlueZ - Bluetooth protocol stack for Linux
  *
- *  Copyright (C) 2006-2007  Nokia Corporation
- *  Copyright (C) 2004-2009  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2006-2010  Nokia Corporation
+ *  Copyright (C) 2004-2010  Marcel Holtmann <marcel@holtmann.org>
  *
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -1535,7 +1535,7 @@ static void handle_setconfiguration_req(struct unix_client *client,
 
 	if (req->codec.seid != client->seid) {
 		error("Unable to set configuration: seid %d not opened",
-				client->seid);
+				req->codec.seid);
 		goto failed;
 	}
 
