@@ -2,8 +2,8 @@
  *
  *  BlueZ - Bluetooth protocol stack for Linux
  *
- *  Copyright (C) 2006-2007  Nokia Corporation
- *  Copyright (C) 2004-2008  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2006-2010  Nokia Corporation
+ *  Copyright (C) 2004-2010  Marcel Holtmann <marcel@holtmann.org>
  *
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -43,6 +43,7 @@
 
 #include <gdbus.h>
 
+#include "hcid.h"
 #include "dbus-common.h"
 #include "logging.h"
 #include "adapter.h"
@@ -60,37 +61,21 @@ const char *manager_get_base_path(void)
 	return base_path;
 }
 
-int manager_update_adapter(uint16_t dev_id)
+void manager_update_svc(const bdaddr_t *bdaddr, uint8_t svc)
 {
-	struct btd_adapter *adapter;
+	GSList *l;
+	bdaddr_t src;
 
-	adapter = manager_find_adapter_by_id(dev_id);
-	if (!adapter)
-		return -EINVAL;
+	for (l = adapters; l != NULL; l = l->next) {
+		struct btd_adapter *adapter = l->data;
 
-	return adapter_update(adapter);
-}
+		adapter_get_address(adapter, &src);
 
-int manager_get_adapter_class(uint16_t dev_id, uint8_t *cls)
-{
-	struct btd_adapter *adapter;
+		if (bacmp(bdaddr, BDADDR_ANY) != 0 && bacmp(bdaddr, &src) != 0)
+			continue;
 
-	adapter = manager_find_adapter_by_id(dev_id);
-	if (!adapter)
-		return -EINVAL;
-
-	return adapter_get_class(adapter, cls);
-}
-
-int manager_set_adapter_class(uint16_t dev_id, uint8_t *cls)
-{
-	struct btd_adapter *adapter;
-
-	adapter = manager_find_adapter_by_id(dev_id);
-	if (!adapter)
-		return -EINVAL;
-
-	return adapter_set_class(adapter, cls);
+		adapter_update(adapter, svc);
+	}
 }
 
 static inline DBusMessage *invalid_args(DBusMessage *msg)
@@ -107,50 +92,6 @@ static inline DBusMessage *no_such_adapter(DBusMessage *msg)
 			"No such adapter");
 }
 
-static int find_by_address(const char *str)
-{
-	struct hci_dev_list_req *dl;
-	struct hci_dev_req *dr;
-	bdaddr_t ba;
-	int i, sk;
-	int devid = -1;
-
-	sk = socket(AF_BLUETOOTH, SOCK_RAW, BTPROTO_HCI);
-	if (sk < 0)
-		return -1;
-
-	dl = g_malloc0(HCI_MAX_DEV * sizeof(*dr) + sizeof(*dl));
-
-	dl->dev_num = HCI_MAX_DEV;
-	dr = dl->dev_req;
-
-	if (ioctl(sk, HCIGETDEVLIST, dl) < 0)
-		goto out;
-
-	dr = dl->dev_req;
-	str2ba(str, &ba);
-
-	for (i = 0; i < dl->dev_num; i++, dr++) {
-		struct hci_dev_info di;
-
-		if (hci_devinfo(dr->dev_id, &di) < 0)
-			continue;
-
-		if (hci_test_bit(HCI_RAW, &di.flags))
-			continue;
-
-		if (!bacmp(&ba, &di.bdaddr)) {
-			devid = dr->dev_id;
-			break;
-		}
-	}
-
-out:
-	g_free(dl);
-	close(sk);
-	return devid;
-}
-
 static DBusMessage *default_adapter(DBusConnection *conn,
 					DBusMessage *msg, void *data)
 {
@@ -179,7 +120,6 @@ static DBusMessage *find_adapter(DBusConnection *conn,
 {
 	DBusMessage *reply;
 	struct btd_adapter *adapter;
-	struct hci_dev_info di;
 	const char *pattern;
 	int dev_id;
 	const gchar *path;
@@ -194,22 +134,13 @@ static DBusMessage *find_adapter(DBusConnection *conn,
 		path = adapter_any_get_path();
 		if (path != NULL)
 			goto done;
-		dev_id = -1;
-	} if (!strncmp(pattern, "hci", 3) && strlen(pattern) >= 4)
+		return no_such_adapter(msg);
+	} else if (!strncmp(pattern, "hci", 3) && strlen(pattern) >= 4) {
 		dev_id = atoi(pattern + 3);
-	else
-		dev_id = find_by_address(pattern);
+		adapter = manager_find_adapter_by_id(dev_id);
+	} else
+		adapter = manager_find_adapter_by_address(pattern);
 
-	if (dev_id < 0)
-		return no_such_adapter(msg);
-
-	if (hci_devinfo(dev_id, &di) < 0)
-		return no_such_adapter(msg);
-
-	if (hci_test_bit(HCI_RAW, &di.flags))
-		return no_such_adapter(msg);
-
-	adapter = manager_find_adapter_by_id(dev_id);
 	if (!adapter)
 		return no_such_adapter(msg);
 
@@ -233,7 +164,6 @@ static DBusMessage *list_adapters(DBusConnection *conn,
 	DBusMessageIter array_iter;
 	DBusMessage *reply;
 	GSList *l;
-	uint16_t dev_id;
 
 	reply = dbus_message_new_method_return(msg);
 	if (!reply)
@@ -246,17 +176,8 @@ static DBusMessage *list_adapters(DBusConnection *conn,
 
 	for (l = adapters; l; l = l->next) {
 		struct btd_adapter *adapter = l->data;
-		struct hci_dev_info di;
 		const gchar *path = adapter_get_path(adapter);
 
-		dev_id = adapter_get_dev_id(adapter);
-
-		if (hci_devinfo(dev_id, &di) < 0)
-			continue;
-
-		if (hci_test_bit(HCI_RAW, &di.flags))
-			continue;
-
 		dbus_message_iter_append_basic(&array_iter,
 					DBUS_TYPE_OBJECT_PATH, &path);
 	}
@@ -290,6 +211,10 @@ static DBusMessage *get_properties(DBusConnection *conn,
 	array = g_new0(char *, g_slist_length(adapters) + 1);
 	for (i = 0, list = adapters; list; list = list->next, i++) {
 		struct btd_adapter *adapter = list->data;
+
+		if (!adapter_is_ready(adapter))
+			continue;
+
 		array[i] = (char *) adapter_get_path(adapter);
 	}
 	dict_append_array(&dict, "Adapters", DBUS_TYPE_OBJECT_PATH, &array, i);
@@ -336,6 +261,10 @@ static void manager_update_adapters(void)
 	array = g_new0(char *, g_slist_length(adapters) + 1);
 	for (i = 0, list = adapters; list; list = list->next, i++) {
 		struct btd_adapter *adapter = list->data;
+
+		if (!adapter_is_ready(adapter))
+			continue;
+
 		array[i] = (char *) adapter_get_path(adapter);
 	}
 
@@ -367,6 +296,9 @@ static void manager_remove_adapter(struct btd_adapter *adapter)
 			DBUS_TYPE_INVALID);
 
 	adapter_remove(adapter);
+
+	if (adapters == NULL)
+		btd_start_exit_timer();
 }
 
 void manager_cleanup(DBusConnection *conn, const char *path)
@@ -395,6 +327,17 @@ static gint adapter_path_cmp(gconstpointer a, gconstpointer b)
 	return strcmp(adapter_path, path);
 }
 
+static gint adapter_cmp(gconstpointer a, gconstpointer b)
+{
+	struct btd_adapter *adapter = (struct btd_adapter *) a;
+	const bdaddr_t *bdaddr = b;
+	bdaddr_t src;
+
+	adapter_get_address(adapter, &src);
+
+	return bacmp(&src, bdaddr);
+}
+
 static gint adapter_address_cmp(gconstpointer a, gconstpointer b)
 {
 	struct btd_adapter *adapter = (struct btd_adapter *) a;
@@ -404,16 +347,24 @@ static gint adapter_address_cmp(gconstpointer a, gconstpointer b)
 
 	adapter_get_address(adapter, &bdaddr);
 	ba2str(&bdaddr, addr);
-	
+
 	return strcmp(addr, address);
 }
 
 struct btd_adapter *manager_find_adapter(const bdaddr_t *sba)
 {
 	GSList *match;
-	char address[18];
 
-	ba2str(sba, address);
+	match = g_slist_find_custom(adapters, sba, adapter_cmp);
+	if (!match)
+		return NULL;
+
+	return match->data;
+}
+
+struct btd_adapter *manager_find_adapter_by_address(const char *address)
+{
+	GSList *match;
 
 	match = g_slist_find_custom(adapters, address, adapter_address_cmp);
 	if (!match)
@@ -449,34 +400,33 @@ GSList *manager_get_adapters(void)
 	return adapters;
 }
 
-static void manager_add_adapter(struct btd_adapter *adapter)
+void manager_add_adapter(const char *path)
 {
-	const gchar *path = adapter_get_path(adapter);
-
-	adapters = g_slist_append(adapters, adapter);
-
 	g_dbus_emit_signal(connection, "/",
 			MANAGER_INTERFACE, "AdapterAdded",
 			DBUS_TYPE_OBJECT_PATH, &path,
 			DBUS_TYPE_INVALID);
 
-	if (default_adapter_id < 0) {
-		int new_default = hci_get_route(NULL);
-
-		manager_set_default_adapter(new_default);
-	}
-
 	manager_update_adapters();
+
+	btd_stop_exit_timer();
 }
 
 int manager_register_adapter(int id, gboolean devup)
 {
-	struct btd_adapter *adapter = adapter_create(connection, id, devup);
+	struct btd_adapter *adapter;
 
+	adapter = manager_find_adapter_by_id(id);
+	if (adapter) {
+		error("Unable to register adapter: hci%d already exist", id);
+		return -1;
+	}
+
+	adapter = adapter_create(connection, id, devup);
 	if (!adapter)
 		return -1;
 
-	manager_add_adapter(adapter);
+	adapters = g_slist_append(adapters, adapter);
 
 	return 0;
 }
@@ -501,7 +451,7 @@ int manager_unregister_adapter(int id)
 
 int manager_start_adapter(int id)
 {
-	struct btd_adapter* adapter;
+	struct btd_adapter *adapter;
 	int ret;
 
 	adapter = manager_find_adapter_by_id(id);
@@ -557,3 +507,17 @@ void manager_set_default_adapter(int id)
 			DBUS_TYPE_OBJECT_PATH, &path,
 			DBUS_TYPE_INVALID);
 }
+
+void btd_manager_set_offline(gboolean offline)
+{
+	GSList *l;
+
+	for (l = adapters; l != NULL; l = g_slist_next(l)) {
+		struct btd_adapter *adapter = l->data;
+
+		if (offline)
+			btd_adapter_switch_offline(adapter);
+		else
+			btd_adapter_restore_powered(adapter);
+	}
+}
