@@ -4,7 +4,7 @@
  *
  *  Copyright (C) 2000-2001  Qualcomm Incorporated
  *  Copyright (C) 2002-2003  Maxim Krasnyansky <maxk@qualcomm.com>
- *  Copyright (C) 2002-2009  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2002-2010  Marcel Holtmann <marcel@holtmann.org>
  *
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -413,6 +413,8 @@ static void return_link_keys(int dev, bdaddr_t *sba, void *ptr)
 
 		info("return_link_keys (sba=%s, dba=%s)", sa, da);
 
+		hcid_dbus_returned_link_key(sba, &dba);
+
 		ptr += 22;
 	}
 }
@@ -651,6 +653,16 @@ static void inquiry_complete(bdaddr_t *local, uint8_t status, gboolean periodic)
 	adapter_set_state(adapter, state);
 }
 
+static inline void remote_features_notify(int dev, bdaddr_t *sba, void *ptr)
+{
+	evt_remote_host_features_notify *evt = ptr;
+
+	if (evt->features[0] & 0x01)
+		hcid_dbus_set_legacy_pairing(sba, &evt->bdaddr, FALSE);
+	else
+		hcid_dbus_set_legacy_pairing(sba, &evt->bdaddr, TRUE);
+}
+
 static inline void cmd_status(int dev, bdaddr_t *sba, void *ptr)
 {
 	evt_cmd_status *evt = ptr;
@@ -985,6 +997,10 @@ static gboolean io_security_event(GIOChannel *chan, GIOCondition cond, gpointer
 		remote_features_information(dev, &di->bdaddr, ptr);
 		break;
 
+	case EVT_REMOTE_HOST_FEATURES_NOTIFY:
+		remote_features_notify(dev, &di->bdaddr, ptr);
+		break;
+
 	case EVT_INQUIRY_COMPLETE:
 		evt = (evt_cmd_status *) ptr;
 		inquiry_complete(&di->bdaddr, evt->status, FALSE);
